# -*- coding: utf-8 -*-

"""A library that provides constants for the `iland.Api` object."""

#: Base URL to the iland cloud API.
BASE_URL = 'https://api.ilandcloud.com/iland-core/rest/v1'

#: Access token URL.
ACCESS_URL = \
    'https://ecs.ilandcloud.com/auth/realms/iland-core/tokens/grants/access'

#: Refresh token URL.
REFRESH_URL = \
    'https://ecs.ilandcloud.com/auth/realms/iland-core/tokens/refresh'
